@@ -1,5 +1,5 @@
 package DBIx::Class::DeploymentHandler::DeployMethod::SQL::Translator;
-$DBIx::Class::DeploymentHandler::DeployMethod::SQL::Translator::VERSION = '0.002223';
+$DBIx::Class::DeploymentHandler::DeployMethod::SQL::Translator::VERSION = '0.002224';
 use Moose;
 
 # ABSTRACT: Manage your SQL and Perl migrations in nicely laid out directories
@@ -235,7 +235,7 @@ sub _run_sql_array {
   my ($self, $sql) = @_;
   my $storage = $self->storage;
 
-  $sql = [ _split_sql_chunk( @$sql ) ];
+  $sql = [ $self->_split_sql_chunk( @$sql ) ];
 
   Dlog_trace { "Running SQL $_" } $sql;
   foreach my $line (@{$sql}) {
@@ -252,29 +252,120 @@ sub _run_sql_array {
   return join "\n", @$sql
 }
 
+my %STORAGE2FEATURE = (
+  SQLServer => {
+    txn => qr/begin\s+transaction\b/i,
+    comment => {
+      DD => 1, # --
+      HASH => 1,
+      SSTAR => 1, # /* */
+      DS => 1, # //
+      PERCENT => 1,
+    },
+  },
+  Sybase => {
+    txn => qr/begin\s+transaction\b/i,
+    comment => {
+      DD => 1,
+      SSTAR => 1,
+      DS => 1,
+      PERCENT => 1,
+    },
+  },
+  SQLite => {
+    txn => qr/begin\b/i,
+    comment => {
+      DD => 1,
+      HASH => 1,
+    },
+  },
+  MySQL => {
+    txn => qr/(begin\b|start\s+transaction\b)/i,
+    comment => {
+      DD => 1,
+      HASH => 1,
+      SS => 1,
+    },
+  },
+  Oracle => {
+    comment => {
+      DD => 1,
+      HASH => 1,
+      SS => 1,
+    },
+  },
+  Pg => {
+    txn => qr/begin\b/i,
+    chunk => sub {
+      my ($c) = @_;
+      my @ret;
+      my $accumulator = '';
+      while (length $c) {
+        if ($c =~ s/\A([^\$]*?);//s) {
+          $accumulator .= $1;
+          push @ret, $accumulator;
+          $accumulator = '';
+        } elsif (
+          $c =~ s/\A(
+            .*?
+            ( \$ [^\$]* \$ )
+          )//xs
+        ) {
+          # got a $...$ .. $...$ chunk
+          $accumulator .= $1;
+          my $anchor = $2;
+          $c =~ s/\A(
+            .*?
+            \Q$anchor\E
+          )//xs;
+          $accumulator .= $1;
+        } elsif ($c =~ s/\A\s*\z//s) {
+          push @ret, $accumulator;
+          $accumulator = '';
+        } else {
+          die "SQL splitting error: $accumulator $c (@ret)";
+        }
+      }
+      @ret;
+    },
+    comment => {
+      DD => 1,
+      HASH => 1,
+    },
+  },
+);
+
 # split a chunk o' SQL into statements
 sub _split_sql_chunk {
-    my @sql = map { split /;\n/, $_ } @_;
-
-    for ( @sql ) {
-        # strip transactions
-        s/^(?:BEGIN|BEGIN TRANSACTION|COMMIT).*//mgi;
-
-        # trim whitespaces
-        s/^\s+//gm;
-        s/\s+$//gm;
-
-        # remove comments
-        s/^--.*//gm;
-
-        # remove blank lines
-        s/^\n//gm;
-
-        # put on single line
-        s/\n/ /g;
-    }
-
-    return grep $_, @sql;
+  my $self = shift;
+  my @sql = map { $_.'' } @_; # copy
+  my $storage_class = ref $self->storage;
+  $storage_class =~ s/.*://;
+  my $feature = $STORAGE2FEATURE{$storage_class} || $STORAGE2FEATURE{MySQL};
+  for ( @sql ) {
+    # strip transactions
+    my $txn = $feature->{txn};
+    s/^\s*($txn|COMMIT\b).*//mgi if $txn;
+    # remove comments
+    my $comment = $feature->{comment};
+    s{--.*}{}gm if $comment->{DD};
+    s{/\* .*? \*/}{}xs if $comment->{SS};
+    s{//.*}{}gm if $comment->{DS};
+    s{#.*}{}gm if $comment->{HASH};
+    s{%.*}{}gm if $comment->{PERCENT};
+  }
+  my $chunk = $feature->{chunk} || sub { split /;\n/, $_[0] };
+  @sql = map $chunk->($_), @sql;
+  for ( @sql ) {
+    # trim whitespace
+    s/^\s+//gm;
+    s/\s+$//gm;
+    # remove blank lines
+    s/^\n//gm;
+    # put on single line
+    s/\n/ /g;
+  }
+  return grep $_, @sql;
 }
 
 sub _run_sql {
@@ -553,6 +644,7 @@ sub _prepare_install {
       }
     }
     open my $file, q(>), $filename;
+    binmode $file;
     print {$file} join ";\n", @$sql, '';
     close $file;
   }
@@ -674,6 +766,7 @@ sub _prepare_changegrade {
     }
 
     open my $file, q(>), $diff_file;
+    binmode $file;
     print {$file} join ";\n", @{$self->_sqldiff_from_yaml($from_version, $to_version, $db, $direction)};
     close $file;
   }
@@ -686,7 +779,7 @@ sub _read_sql_file {
    local $/ = undef;  #sluuuuuurp
 
   open my $fh, '<', $file;
-  return [ _split_sql_chunk( <$fh> ) ];
+  return [ $self->_split_sql_chunk( <$fh> ) ];
 }
 
 sub downgrade_single_step {
@@ -758,6 +851,7 @@ sub prepare_protoschema {
   }
 
   open my $file, q(>), $filename;
+  binmode $file;
   print {$file} $yml;
   close $file;
 }
